// $Id: MainWindow.cpp,v 1.26 2011/11/22 14:51:50 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file MainWindow.cc
\brief customized QVBox to recieve close event
\author Hugo Pereira
\version $Revision: 1.26 $
\date $Date: 2011/11/22 14:51:50 $
*/

#include "Application.h"
#include "CustomToolBar.h"
#include "IconEngine.h"
#include "Icons.h"
#include "Menu.h"
#include "QtUtil.h"
#include "StatusBar.h"
#include "MainWindow.h"
#include "ServerXmlDef.h"
#include "Singleton.h"
#include "TreeView.h"
#include "XmlOptions.h"

#include <QtGui/QLayout>

//___________________________________________
const QString MainWindow::VERBOSITY_HIGH = "high";
const QString MainWindow::VERBOSITY_LOW = "low";

//___________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    verbosity_( LOW )

{
    setOptionName( "MAIN_WINDOW" );
    Debug::Throw( "MainWindow::MainWindow.\n" );

    // install actions
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    main->setLayout(  new QVBoxLayout() );
    main->layout()->setMargin(0);
    main->layout()->setSpacing(5);
    setCentralWidget( main );

    // toolbar
    toolbar_ = new CustomToolBar( "Main Toolbar", this, "TOOLBAR" );
    toolBar().addAction( &resetAction() );
    toolBar().addAction( &unlockAction() );
    toolBar().addSeparator();
    toolBar().addAction( &raiseAction() );
    toolBar().addAction( &abortAction() );


    // command list
    main->layout()->addWidget( list_ = new TreeView( main ) );
    list().setModel( &model_ );
    list().setSortingEnabled( true );
    list().setRootIsDecorated( true );
    list().setSelectionMode( TreeView::ContiguousSelection );
    list().setOptionName( "LIST" );

    list().menu().addAction( &resetAction() );
    list().menu().addAction( &unlockAction() );
    list().menu().addSeparator();
    list().menu().addAction( &raiseAction() );
    list().menu().addAction( &abortAction() );
    list().menu().addSeparator();
    list().menu().addAction( &toolBar().visibilityAction() );

    connect( list().selectionModel(), SIGNAL( selectionChanged ( const QItemSelection &, const QItemSelection & ) ), SLOT( _updateActions() ) );
    connect( &list(), SIGNAL( activated( const QModelIndex& ) ), SLOT( _requestRaise() ) );

    connect( &model_, SIGNAL( layoutAboutToBeChanged() ), SLOT( _storeSelection() ) );
    connect( &model_, SIGNAL( layoutAboutToBeChanged() ), SLOT( _storeExpandedItems() ) );

    connect( &model_, SIGNAL( layoutChanged() ), SLOT( _restoreSelection() ) );
    connect( &model_, SIGNAL( layoutChanged() ), SLOT( _restoreExpandedItems() ) );

    // state frame
    setStatusBar( statusbar_ = new StatusBar( this ) );
    statusBar().addLabel( 1, true );
    statusBar().addClock();

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );

    connect( &application, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();
    _updateActions();

}

//_______________________________________
void MainWindow::append( const ServerCommandModel::Set& commands )
{

    Debug::Throw( "MainWindow::append.\n" );

    // update state frame
    ServerCommandModel::Set added_commands;
    ServerCommandModel::Set removed_commands;

    for( ServerCommandModel::Set::iterator iter = commands.begin(); iter != commands.end(); ++iter )
    {

        const SERVER::ServerCommand& command = *iter;

        // print command (XML format)
        QDomDocument document;
        QDomElement top = document.appendChild( document.createElement( SERVER_XML::TRANSMISSION ) ).toElement();
        top.appendChild( iter->domElement( document ) );
        Debug::Throw() << "MainWindow::append - " << document.toString() << endl;

        QString buffer;
        QTextStream what( &buffer, QIODevice::WriteOnly );
        what
            << "recieved " << command.commandName()
            << " from " << command.id().name();
        statusBar().label().setTextAndUpdate( buffer );

        if( verbosity_ == LOW && (
            command.command() == SERVER::ServerCommand::RAISE ||
            command.command() == SERVER::ServerCommand::ALIVE ||
            command.command() == SERVER::ServerCommand::DENIED ) )
        { continue; }

        // for KILLED command and low verbosity, removes corresponding item
        if( verbosity_ == LOW && command.command() == SERVER::ServerCommand::KILLED )
        {

            ServerCommandModel::List commands( model().children() );
            for( ServerCommandModel::List::iterator iter = commands.begin(); iter != commands.end(); ++iter )
            { if( iter->id() == command.id() ) removed_commands.insert( *iter ); }
            continue;

        }

        added_commands.insert( command );
    }

    model().remove( removed_commands );
    model().add( added_commands );

    list().resizeColumnToContents( ServerCommandModel::NAME );

    return;
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    event->accept();
}

//___________________________________________________________
void MainWindow::_updateConfiguration( void )
{

    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // resize
    setVerbosity( XmlOptions::get().raw( "VERBOSITY" ) );

    // colors
    bool changed( false );
    changed |= _model().setServerColor( XmlOptions::get().get<QString>( "SERVER_COLOR" ) );
    changed |= _model().setIdentifyColor( XmlOptions::get().get<QString>( "IDENTIFY_COLOR" ) );

}

//________________________________________
void MainWindow::_storeSelection( void )
{

    Debug::Throw( "MainWindow::_storeSelection.\n" );

    // clear
    model().clearSelectedIndexes();

    // retrieve selected indexes in list
    QModelIndexList selected_indexes( list().selectionModel()->selectedIndexes() );
    for( QModelIndexList::iterator iter = selected_indexes.begin(); iter != selected_indexes.end(); ++iter )
    {
        // check column
        if( !iter->column() == 0 ) continue;
        model().setIndexSelected( *iter, true );
    }

}

//________________________________________
void MainWindow::_storeExpandedItems( void )
{

    Debug::Throw( "MainWindow::_storeExpandedItems.\n" );

    // clear
    model().clearExpandedIndexes();

    // retrieve items.
    QModelIndexList indexes( model().indexes() );
    for( QModelIndexList::iterator iter = indexes.begin(); iter != indexes.end(); ++iter )
    { if( list().isExpanded( *iter ) ) model().setIndexExpanded( *iter, true ); }

}

//________________________________________
void MainWindow::_restoreSelection( void )
{

    Debug::Throw( "MainWindow::_restoreSelection.\n" );
    QModelIndexList selected_indexes( model().selectedIndexes() );
    if( selected_indexes.empty() ) list().selectionModel()->clearSelection();
    else {

        list().selectionModel()->select( selected_indexes.front(),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        for( QModelIndexList::const_iterator iter = selected_indexes.begin(); iter != selected_indexes.end(); ++iter )
        { list().selectionModel()->select( *iter, QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

    }

    return;
}

//________________________________________
void MainWindow::_restoreExpandedItems( void )
{

    Debug::Throw( "MainWindow::_restoreExpandedItems.\n" );
    QModelIndexList expanded_indexes( model().expandedIndexes() );
    list().collapseAll();
    for( QModelIndexList::const_iterator iter = expanded_indexes.begin(); iter != expanded_indexes.end(); ++iter )
    { list().setExpanded( *iter, true ); }

}

//_______________________________________
void MainWindow::_updateActions( void )
{

    Debug::Throw( "MainWindow::_updateActions.\n" );
    bool has_selection( !list().selectionModel()->selectedIndexes().empty() );
    raiseAction().setEnabled( has_selection );
    abortAction().setEnabled( has_selection );
    return;
}

//________________________________________
void MainWindow::_requestReset( void )
{
    Debug::Throw( "MainWindow::_requestReset.\n" );
    statusBar().label().setTextAndUpdate( "resetting list ..." );
    model().clear();
    emit requestCommand( SERVER::ServerCommand::IDENTIFY );
}

//________________________________________
void MainWindow::_requestUnlock( void )
{
    Debug::Throw( "MainWindow::_requestUnlock.\n" );
    statusBar().label().setTextAndUpdate( "unlocking registered applications ..." );
    model().clear();
    emit requestCommand( SERVER::ServerCommand::UNLOCK );
    emit requestCommand( SERVER::ServerCommand::IDENTIFY );
}

//________________________________________
void MainWindow::_requestRaise( void )
{
    Debug::Throw( "MainWindow::_requestRaise.\n" );
    ServerCommandModel::List commands( _model().get( list().selectionModel()->selectedRows() ) );
    for( ServerCommandModel::List::iterator iter = commands.begin(); iter != commands.end(); ++iter )
    { emit requestCommand( SERVER::ServerCommand( iter->id(), SERVER::ServerCommand::RAISE ) ); }
}

//________________________________________
void MainWindow::_requestAbort( void )
{
    Debug::Throw( "MainWindow::_requestAbort.\n" );

    ServerCommandModel::List commands( _model().get( list().selectionModel()->selectedRows() ) );
    for( ServerCommandModel::List::iterator iter = commands.begin(); iter != commands.end(); ++iter )
    { emit requestCommand( SERVER::ServerCommand( iter->id(), SERVER::ServerCommand::ABORT ) ); }
    _requestReset();

}

//________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installAction.\n" );

    addAction( resetAction_ = new QAction( IconEngine::get( ICONS::RESET ), "&Reload", this ) );
    resetAction_->setShortcut( Qt::Key_F5 );
    connect( resetAction_, SIGNAL( triggered() ), SLOT( _requestReset() ) );

    addAction( unlockAction_ = new QAction( IconEngine::get( ICONS::UNLOCK ), "&Unlock", this ) );
    connect( unlockAction_, SIGNAL( triggered() ), SLOT( _requestUnlock() ) );

    // actions
    addAction( raiseAction_ = new QAction( IconEngine::get( ICONS::RAISE ), "&Raise", this ) );
    raiseAction_->setToolTip( "Raise selected applications" );
    connect( raiseAction_, SIGNAL( triggered() ), SLOT( _requestRaise() ) );


    addAction( abortAction_ = new QAction( IconEngine::get( ICONS::ABORT ), "&Abort", this ) );
    abortAction_->setToolTip( "Abort selected applications" );
    connect( abortAction_, SIGNAL( triggered() ), SLOT( _requestAbort() ) );

}
